#include "PacketLayer.h"

PacketLayer::PacketLayer (NetworkInterface& iface)
: m_iface	(iface)
{
	for (unsigned short i=0; i<Packet::PROTO_MAXIMUM; i++)
		m_infilter.push_back (0);
}

PacketLayer::~PacketLayer ()
{
}

void PacketLayer::registerAllProtocols ()
{
	for (unsigned short i=0; i<Packet::PROTO_MAXIMUM; i++)
		m_infilter[i] += 1;
}

void PacketLayer::unregisterAllProtocols ()
{
	for (unsigned short i=0; i<Packet::PROTO_MAXIMUM; i++)
		m_infilter[i] -= 1;
}

void PacketLayer::registerProtocol (Packet::PROTOCOL proto)
{
	m_infilter[proto] += 1;
}

void PacketLayer::unregisterProtocol (Packet::PROTOCOL proto)
{
	assert (m_infilter[proto] > 0);
	m_infilter[proto] = m_infilter[proto] - 1;
}

Packet* PacketLayer::read ()
{
	//
	// read the frame
	//

	Frame* frame  = m_iface.read ();
	if (frame == NULL) return NULL;

	//
	// parse all protocols
	//

	frame->payloadpacket->parsePacket		();
	frame->payloadpacket->createNextLayer	();	

	//
	// do we need this packet according to the in-filter protocol filtering?
	//

	Packet*	packet = frame->payloadpacket;

	for (Packet* current=packet; current!=NULL; current=current->getNextPacket()) {
		
		Packet::PROTOCOL proto = current->getProtocol ();
		if (m_infilter[proto] > 0) return packet;

	}

	//
	// no filter set for any of the protocols
	// so the frame is worthless
	// TODO: delete packet, frame, etc ...

	return NULL;
}

