#include "Log.h"

Log*	Log::object  = NULL;
bool	Log::destroy = false;

Log* Log::instance ()
{
	static Guard guard;

	if (object == NULL) {
		object  = new Log ();
		destroy = true;
	}
	
	return object;
}

void Log::reference (Log* log)
{
	destroy = false;
	object  = log;
}

Log::Log() 
{
}

Log::~Log()
{
}

void Log::error(string err)
{
	Log::instance()->output (err, LogLevel::TYPE_ERROR);
}

void Log::warning(string warn)
{
	Log::instance()->output (warn, LogLevel::TYPE_WARNING);
}

void Log::info(string inf)
{
	Log::instance()->output (inf, LogLevel::TYPE_INFO);
}

void Log::debug(string dbg)
{
	Log::instance()->output (dbg, LogLevel::TYPE_DEBUG);
}

void Log::output(string str, LogLevel::LOG_LEVEL type)
{
	boost::mutex::scoped_lock scopedLock (m_outputmutex);

	// no output active?
	if (m_outputters.size () <= 0)
		return;

	// check if output is desired with the current log level
	if ((m_logLevel & type) != type) 
		return;

	LogOutputter::OUTPUT_OBJ outobj;

	// get the current date-time
	outobj.date	= Helper::getDate ();
	outobj.time	= Helper::getTime ();

	outobj.type	= type;
	outobj.text	= Helper::trim (str);

	OUTPUTTER_LIST::iterator begin	= m_outputters.begin	();
	OUTPUTTER_LIST::iterator end	= m_outputters.end		();

	//
	// finally output the item in the registered LogOutputter's
	//

	for ( ; begin != end; begin++)
		(*begin)->output (&outobj);
}

void Log::setLogLevel(LogLevel::LOG_LEVEL level)
{
	Log::instance()->m_logLevel = level;
}

LogLevel::LOG_LEVEL Log::getLogLevel ()
{
	return Log::instance()-> m_logLevel;
}

void Log::addLogOutput (LogOutputter* out)
{
	boost::mutex::scoped_lock scopedLock (Log::instance()->m_outputmutex);
	Log::instance()->m_outputters.push_back (out);
}

void Log::removeLogOutput (LogOutputter* outputter)
{
	Log* logger = Log::instance ();
	boost::mutex::scoped_lock scopedLock (logger->m_outputmutex);
	
	
	OUTPUTTER_LIST_ITERATOR item = find (logger->m_outputters.begin(), logger->m_outputters.end(), outputter);
	if (item != logger->m_outputters.end())
		logger->m_outputters.erase (item);
}
