#ifndef __LOG_H
#define __LOG_H

#include <iostream>
#include <string>
#include <iomanip> 
#include <cassert>
#include <ostream>
#include <sstream>
#include <list>
#include <algorithm>
#include <boost/thread/mutex.hpp>
#include "base/common/Helper.h"
#include "base/common/LogLevel.h"
#include "base/common/LogOutputter.h"

using std::find;
using std::list;
using std::setfill;
using std::setw;
using std::cout;
using std::string;
using std::ostream;
using std::ostringstream;

class Outputter;

class Log {
public:

	static void						reference				(Log* factory);
	static Log*						instance				();

	static void 					debug					(string dbg	);
	static void 					info					(string inf	);
	static void 					warning					(string warn);
	static void 					error					(string err	);

	static void						setLogLevel				(LogLevel::LOG_LEVEL level);
	static LogLevel::LOG_LEVEL		getLogLevel				();
	static void						addLogOutput			(LogOutputter*	outputter);
	static void						removeLogOutput			(LogOutputter*	outputter);

private:
									Log						();
									~Log					();

	class Guard {
		public: ~Guard() {
			if (Log::object != NULL && destroy)
				delete Log::object;
			}
	};

	friend class					Guard;
	static Log*						object;
	static bool						destroy;

	typedef list<LogOutputter*>		 OUTPUTTER_LIST;
	typedef OUTPUTTER_LIST::iterator OUTPUTTER_LIST_ITERATOR;
	OUTPUTTER_LIST					 m_outputters;

	LogLevel::LOG_LEVEL				m_logLevel;
	boost::mutex					m_outputmutex;
	
	void							output					(string str, LogLevel::LOG_LEVEL type);
};

#endif // __LOG_H

