#ifndef __CONFIGURATION_H
#define __CONFIGURATION_H

#include "triggerconf.h"
#include <string>
#include <iostream>
#include "base/module/DistackModuleInterface.h"
#include "base/common/Helper.h"
#include "base/common/Log.h"

using std::string;
using std::cout;

class Configuration 
{
public:
	
	static void					init					(string file);
	static Configuration*		instance				();

	typedef enum _NETWORK_TRACE_SOURCE {
		NETWORK_TRACE_SOURCE_UNDEFINED,
		NETWORK_TRACE_SOURCE_INTERFACE,
		NETWORK_TRACE_SOURCE_TRACEFILE,
	} NETWORK_TRACE_SOURCE; 

	LogLevel::LOG_LEVEL			getLogLevel				();

	NETWORK_TRACE_SOURCE		getFrameSourceType		();

	bool						getChecksumCheck		();
	string						getNetworkInterface		();
	string						getTraceFile			();
	double						getTraceReplaySpeedup	();
	bool						getNetworkPromiscuous	();
	unsigned int				getNetworkCaptureLen	();
	Triggerconf::STRING_LIST	getRemoteNeighbors		();
	string						getRemoteMessaging		();
	unsigned int				getQueueMaxFrames		();
	unsigned int				getQueueMaxBytes		();
	Triggerconf::STRING_LIST	getAvailableModules		();
	ModuleConfiguration			getModuleConfiguration	(string module);
	Triggerconf::STRING_LIST	getChannels				();
	Triggerconf::STRING_LIST	getChannels				(unsigned short stage);
	Triggerconf::STRING_LIST	getChannelItems			(string channel);

	static const string			DELIM;
	static const string			ATTRIBUTE_LIBRARY;
	static const string			ATTRIBUTE_STAGE;

	static const string			PATH_GENERAL;
	static const string			PATH_GENERAL_LOG;
	static const string			PATH_GENERAL_LOG_LOGLEVEL;
	static const string			PATH_GENERAL_NETWORK;
	static const string			PATH_GENERAL_NETWORK_CHECKSUMCHECK;
	static const string			PATH_GENERAL_NETWORK_INTERFACE;
	static const string			PATH_GENERAL_NETWORK_TRACEFILE;
	static const string			PATH_GENERAL_NETWORK_TRACEREPLAYSPEEDUP;
	static const string			PATH_GENERAL_NETWORK_PROMISCUOUS;
	static const string			PATH_GENERAL_NETWORK_CAPTURELENGTH;	
	static const string			PATH_GENERAL_REMOTE;
	static const string			PATH_GENERAL_REMOTE_NEIGHBORS;
	static const string			PATH_GENERAL_REMOTE_MESSAGING;
	static const string			PATH_GENERAL_QUEUE;
	static const string			PATH_GENERAL_QUEUE_MAX_FRAME_LEN;
	static const string			PATH_GENERAL_QUEUE_MAX_BYTE_LEN;
	static const string			PATH_MODULES;
	static const string			PATH_CHANNELS;

private:
								Configuration			(string filename);
								~Configuration			();

	class Guard {
		public: ~Guard() {
			if (Configuration::object != NULL)
				delete Configuration::object;
			}
	};

	friend class				Guard;
	static Configuration*		object;
	static string				filename;
	Triggerconf					tconf;

};

#endif // __CONFIGURATION_H

