#ifndef __REMOTE_COMM_SYSTEM_H
#define __REMOTE_COMM_SYSTEM_H

#include <string>
#include "base/protocol/IpPacket.h"
#include "base/message/MessagingNode.h"

using std::string;

#define DISTACK_REMOTE_MESSAGING_MAGIC		0xd15fac30 // the distack magic. 4 bytes that nearly looks like distack :)

typedef struct _DISTACK_REMOTE_DATA {
	IP_ADDR			source;
	IP_ADDR			dest;

	unsigned char*	data;
	unsigned long	len;

	_DISTACK_REMOTE_DATA (int length = 0) {
		if (length > 0) data = (unsigned char*) malloc (length);
		else			data = 0;

		len = length;
	}

	void destroy () {
		free (data);
	}
} DISTACK_REMOTE_DATA;

typedef struct _DISTACK_REMOTE_HEADER {
	unsigned long				magic;
	unsigned long				payloadlen;
	IP_ADDR						source;
	IP_ADDR						dest;
	static const unsigned long	headerlen = 16;

	_DISTACK_REMOTE_HEADER () {
		_DISTACK_REMOTE_HEADER (0);
	}

	_DISTACK_REMOTE_HEADER (unsigned long len) {
		magic		= DISTACK_REMOTE_MESSAGING_MAGIC;
		payloadlen	= len;
	}
	
	unsigned long totalLen () {
		return  headerlen + payloadlen;
	}

	bool valid () {
		return (magic == DISTACK_REMOTE_MESSAGING_MAGIC);
	}
} DISTACK_REMOTE_HEADER;

class RemoteMessagingSystem; // forward decl

class RemoteCommSystem {
public:
							RemoteCommSystem			(string name, string description);
	virtual					~RemoteCommSystem			();

	void					setMessagingSystem			(RemoteMessagingSystem* system);		

	//
	// write data to the network. this function must return as fast
	// as possible! If the write takes a long time, the implemented
	// RemoteCommSystem must buffer the data and write it later.
	// The DATA.data buffer is destroyed by the RemoteMessagingSystem
	// as soon as the function returns! If doing async writes the 
	// implemented RemoteCommSystem must create a copy of the data!
	virtual bool			write						(DISTACK_REMOTE_DATA data, MessagingNode::SEND_OPTIONS options) = 0;

	//
	// call this function from your ReadCommSystem implementation
	// when you got data read and ready to transfer to upper layers
	void					read						(DISTACK_REMOTE_DATA data);	

	string					getName						();
	string					getDescription				();

private:

	string					m_name;
	string					m_description;
	RemoteMessagingSystem*	m_messagingSystem;

};

#endif // __REMOTE_COMM_SYSTEM_H
