#include "NetworkManager.h"

NetworkManager::NetworkManager ()
{
}

NetworkManager::~NetworkManager ()
{
}

bool NetworkManager::startup ()
{
	Log::debug ("network manager is starting up ...");

	//
	// get network configuration parameters
	//

	Configuration::NETWORK_TRACE_SOURCE source = Configuration::instance()->getFrameSourceType ();

#ifdef OMNET_SIMULATION 
	source = Configuration::NETWORK_TRACE_SOURCE_INTERFACE;
#else
	if (source == Configuration::NETWORK_TRACE_SOURCE_UNDEFINED) {
		Log::error ("undefined network frame source");
		return false;
	}
#endif // OMNET_SIMULATION

	bool			ret			= true;
	string			iface		= Configuration::instance()->getNetworkInterface	();
	string			tracefile	= Configuration::instance()->getTraceFile			();
	bool			promisc		= Configuration::instance()->getNetworkPromiscuous	();
	unsigned int	captureLen	= Configuration::instance()->getNetworkCaptureLen	();

	//
	// set protocol parsing parameters
	//

	// TODO: is this also set in the analyzer modules which have their own compilation of the protocol classes?
	Packet::setChecksumValidation (Configuration::instance()->getChecksumCheck ());

	//
	// create and open the network interface
	//

	Log::debug ("creating interface ...");

	if (source == Configuration::NETWORK_TRACE_SOURCE_INTERFACE){	

#ifdef OMNET_SIMULATION
		m_interface = new NetworkInterfaceOmnetpp ();
#else
		m_interface = new NetworkInterfacePcap (iface, promisc, captureLen);
		Log::debug ("online interface created");
#endif // OMNET_SIMULATION

	} else if (source == Configuration::NETWORK_TRACE_SOURCE_TRACEFILE) {
	
#ifdef OMNET_SIMULATION
		Log::error ("can not use offline source in Omnet simulation");
		return false;
#else
		double speedup = Configuration::instance()->getTraceReplaySpeedup ();
		m_interface = new NetworkInterfacePcap (tracefile, speedup);
		Log::debug ("offline interface created");
#endif // OMNET_SIMULATION

	} else {

		assert (false);
		return false;

	}

	Log::debug ("opening interface ...");
	ret = m_interface->open ();									
	if (! ret) {
		Log::error ("opening interface " + iface + " failed");
		return false;
	}

	//
	// create the frame-buffer and set it on top of the network interface
	//

#ifndef OMNET_SIMULATION
	
	Log::debug ("creating frame-buffer ...");

	m_frameBuffer = new FrameBuffer (*m_interface);

	unsigned int	maxFrames	=	Configuration::instance()->getQueueMaxFrames ();
	unsigned int	maxBytes	=	Configuration::instance()->getQueueMaxBytes ();
	m_frameBuffer->setMaxFrames(maxFrames);
	m_frameBuffer->setMaxBytes(maxBytes);

	m_interface->setFrameBuffer		(m_frameBuffer);

#endif // OMNET_SIMULATION

	return true;
}

bool NetworkManager::shutdown ()
{
	Log::debug ("network manager ist shutting down ...");

#ifndef OMNET_SIMULATION

	m_interface->setFrameBuffer (NULL);
	delete m_frameBuffer;

#endif // OMNET_SIMULATION

	delete m_interface;

	return true; 
}

FrameBuffer* NetworkManager::getFrameSource ()
{
	return m_frameBuffer;
}

NetworkInterface* NetworkManager::getNetworkInterface ()
{
	return m_interface;
}
